# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2005 Dan Williams <dcbw@redhat.com> and Red Hat, Inc.

try:
    import sqlite
except ImportError, e:
    import pysqlite2._sqlite as sqlite

import sys
sys.path.append('/usr/share/plague/server')
from DBManager import ResultSet

CONFIG_LOCATION = "/etc/plague/server/"

class User:
    def __init__(self, email, guest):
        if not email:
            raise Exception
        self.email = email
        self.guest = guest
        self.own_jobs = False
        self.job_admin = False
        self.user_admin = False
        self.server_admin = False

def get_userdb_dbcx():
    try:
        dbcx = sqlite.connect(CONFIG_LOCATION + "userdb", timeout=4)
        curs = dbcx.cursor()
        return (dbcx, curs)
    except sqlite.DatabaseError, e:
        print "sqlite DatabaseError: %s" % str(e)
        return (None, None)

class Authenticator:
    """
    Talks to a database of users & capabilities
    """

    def __init__(self, cfg):
        self._cfg = cfg
        (dbcx, curs) = get_userdb_dbcx()
        if not dbcx or not curs:
            print "Unable to open the user database.  Exiting..."
            os._exit(1)

        # Ensure the table exists in the database
        create = False
        try:
            curs.execute('SELECT * FROM users')
            dbcx.commit()
        except sqlite.DatabaseError, e:
            create = True

        if create:
            curs.execute('CREATE TABLE users (' \
                    'email VARCHAR(50), '       \
                    'own_jobs BOOLEAN, '        \
                    'job_admin BOOLEAN, '       \
                    'user_admin BOOLEAN, '    \
                    'server_admin BOOLEAN)')
            dbcx.commit()
        del curs, dbcx


    def new_authed_user(self, email, client_address):
        if not email:
            return None

        (dbcx, curs) = get_userdb_dbcx()
        user = None
        curs.execute('SELECT * FROM users WHERE email="%s"' % email)
        dbcx.commit()
        item = curs.fetchone()
        if item:
            item = ResultSet(item, curs.description)
            user = User(email, False)
            user.own_jobs = item['own_jobs']
            user.job_admin = item['job_admin']
            user.user_admin = item['user_admin']
            user.server_admin = item['server_admin']
        else:
            if self._cfg.get_bool("UI", "guest_allowed"):
                user = User('guest@guest', True)
        del curs, dbcx
        return user

